@@ -2,7 +2,7 @@
 --
 -- This file is part of Nominatim. (https://nominatim.org)
 --
--- Copyright (C) 2024 by the Nominatim developer community.
+-- Copyright (C) 2025 by the Nominatim developer community.
 -- For a full list of authors see the git log.
 
 -- Trigger functions for the placex table.
@@ -530,6 +530,7 @@ CREATE OR REPLACE FUNCTION insert_addresslines(obj_place_id BIGINT,
   AS $$
 DECLARE
   address_havelevel BOOLEAN[];
+  place_min_distance FLOAT[];
 
   location_isaddress BOOLEAN;
   current_boundary GEOMETRY := NULL;
@@ -545,6 +546,7 @@ BEGIN
   nameaddress_vector := '{}'::int[];
 
   address_havelevel := array_fill(false, ARRAY[maxrank]);
+  place_min_distance := array_fill(1.0, ARRAY[maxrank]);
 
   FOR location IN
     SELECT apl.*, key
@@ -575,6 +577,10 @@ BEGIN
         END IF;
       END IF;
 
+      IF location.isguess and location.distance < place_min_distance[location.rank_address] THEN
+        place_min_distance[location.rank_address] := location.distance;
+      END IF;
+
       INSERT INTO place_addressline (place_id, address_place_id, fromarea,
                                      isaddress, distance, cached_rank_address)
         VALUES (obj_place_id, location.place_id, not location.isguess,
@@ -602,6 +608,16 @@ BEGIN
     -- If this is the first item in the rank, then assume it is the address.
     location_isaddress := not address_havelevel[location.rank_address];
 
+    -- Ignore guessed places when they are too far away compared to similar closer ones.
+    IF location.isguess THEN
+      CONTINUE WHEN not location_isaddress
+                    AND location.distance > 2 * place_min_distance[location.rank_address];
+
+      IF location.distance < place_min_distance[location.rank_address] THEN
+        place_min_distance[location.rank_address] := location.distance;
+      END IF;
+    END IF;
+
     -- Further sanity checks to ensure that the address forms a sane hierarchy.
     IF location_isaddress THEN
       IF location.isguess and current_node_area is not NULL THEN
@@ -638,8 +654,10 @@ BEGIN
 
     -- Add it to the list of search terms
     {% if not db.reverse_only %}
-      nameaddress_vector := array_merge(nameaddress_vector,
-                                        location.keywords::integer[]);
+      IF location.rank_address != 11 AND location.rank_address != 5 THEN
+          nameaddress_vector := array_merge(nameaddress_vector,
+                                            location.keywords::integer[]);
+      END IF;
     {% endif %}
 
     INSERT INTO place_addressline (place_id, address_place_id, fromarea,
@@ -730,7 +748,7 @@ BEGIN
   IF NEW.rank_address between 2 and 27 THEN
     IF (ST_GeometryType(NEW.geometry) in ('ST_Polygon','ST_MultiPolygon') AND ST_IsValid(NEW.geometry)) THEN
       -- Performance: We just can't handle re-indexing for country level changes
-      IF (NEW.rank_address < 26 and st_area(NEW.geometry) < 1)
+      IF (NEW.rank_address < 26 and st_area(NEW.geometry) <= 2)
          OR (NEW.rank_address >= 26 and st_area(NEW.geometry) < 0.01)
       THEN
         -- mark items within the geometry for re-indexing
@@ -778,7 +796,7 @@ BEGIN
   SELECT count(*)>0 FROM pg_tables WHERE tablename = classtable and schemaname = current_schema() INTO result;
   IF result THEN
     EXECUTE 'INSERT INTO ' || classtable::regclass || ' (place_id, centroid) VALUES ($1,$2)' 
-    USING NEW.place_id, ST_Centroid(NEW.geometry);
+    USING NEW.place_id, NEW.centroid;
   END IF;
 
 {% endif %} -- not disable_diff_updates
@@ -872,6 +890,11 @@ BEGIN
   -- Compute a preliminary centroid.
   NEW.centroid := get_center_point(NEW.geometry);
 
+  -- Record the entrance node locations
+  IF NEW.osm_type = 'W' and (NEW.rank_search > 27 or NEW.class IN ('landuse', 'leisure')) THEN
+    PERFORM place_update_entrances(NEW.place_id, NEW.osm_id);
+  END IF;
+
     -- recalculate country and partition
   IF NEW.rank_search = 4 AND NEW.address is not NULL AND NEW.address ? 'country' THEN
     -- for countries, believe the mapped country code,
@@ -960,9 +983,8 @@ BEGIN
           WHERE class = 'place' and rank_address between 1 and 23
                 and prank.address_rank >= NEW.rank_address
                 and ST_GeometryType(geometry) in ('ST_Polygon','ST_MultiPolygon') -- select right index
-                and geometry && NEW.geometry
-                and geometry ~ NEW.geometry -- needed because ST_Relate does not do bbox cover test
-                and ST_Relate(geometry, NEW.geometry, 'T*T***FF*') -- contains but not equal
+                and ST_Contains(geometry, NEW.geometry)
+                and not ST_Equals(geometry, NEW.geometry)
           ORDER BY prank.address_rank desc LIMIT 1
         LOOP
           NEW.rank_address := location.rank_address + 2;
@@ -983,9 +1005,8 @@ BEGIN
                 and rank_address between 1 and 25 -- select right index
                 and ST_GeometryType(geometry) in ('ST_Polygon','ST_MultiPolygon') -- select right index
                 and prank.address_rank >= NEW.rank_address
-                and geometry && NEW.geometry
-                and geometry ~ NEW.geometry -- needed because ST_Relate does not do bbox cover test
-                and ST_Relate(geometry, NEW.geometry, 'T*T***FF*') -- contains but not equal
+                and ST_Contains(geometry, NEW.geometry)
+                and not ST_Equals(geometry, NEW.geometry)
           ORDER BY prank.address_rank desc LIMIT 1
         LOOP
           NEW.rank_address := location.rank_address + 2;
